@@ -1,29 +1,27 @@
 package Parse::HTTP::UserAgent::Base::Accessors;
 use strict;
 use warnings;
-use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.39';
-
-#TODO: new accessors
-#wap
-#mobile
-#device
-#tablet
+our $VERSION = '0.11';
 
 BEGIN {
     my @simple = qw(
-        name
-        unknown
+        device
         generic
-        os
         lang
-        strength
-        parser
+        mobile
+        name
         original_name
         original_version
+        os
+        parser
         robot
+        strength
+        tablet
+        touch
+        unknown
+        wap
     );
 
     my @multi = qw(
@@ -97,9 +95,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Parse::HTTP::UserAgent::Base::Accessors - Available accessors
+Parse::HTTP::UserAgent::Base::Accessors
+
+=head1 VERSION
+
+version 0.41
 
 =head1 SYNOPSIS
 
@@ -112,15 +116,18 @@ Parse::HTTP::UserAgent::Base::Accessors - Available accessors
 
 =head1 DESCRIPTION
 
-This document describes version C<0.39> of C<Parse::HTTP::UserAgent::Base::Accessors>
-released on C<2 December 2013>.
-
 Ther methods can be used to access the various parts of the parsed structure.
 
+=head1 NAME
+
+Parse::HTTP::UserAgent::Base::Accessors - Available accessors
+
 =head1 ACCESSORS
 
 The parts of the parsed structure can be accessed using these methods:
 
+=head2 device
+
 =head2 dotnet
 
 =head2 extras
@@ -129,6 +136,8 @@ The parts of the parsed structure can be accessed using these methods:
 
 =head2 lang
 
+=head2 mobile
+
 =head2 mozilla
 
 =head2 name
@@ -145,27 +154,31 @@ The parts of the parsed structure can be accessed using these methods:
 
 =head2 strength
 
+=head2 tablet
+
+=head2 touch
+
 =head2 toolkit
 
 =head2 unknown
 
 =head2 version
 
+=head2 wap
+
 =head1 SEE ALSO
 
 L<Parse::HTTP::UserAgent>.
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak Gursoy <burak@cpan.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 - 2013 Burak Gursoy. All rights reserved.
+This software is copyright (c) 2009 by Burak Gursoy.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.16.2 or,
-at your option, any later version of Perl 5 you may have available.
 =cut
