@@ -1,11 +1,9 @@
 package Parse::HTTP::UserAgent::Base::Parsers;
+$Parse::HTTP::UserAgent::Base::Parsers::VERSION = '0.41';
 use strict;
 use warnings;
-use vars qw( $VERSION );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-$VERSION = '0.39';
-
 sub _extract_dotnet {
     my($self, @args) = @_;
     my @raw  = map { ref($_) eq 'ARRAY' ? @{$_} : $_ } grep { $_ } @args;
@@ -44,7 +42,7 @@ sub _fix_opera {
     }
     $self->_fix_os_lang;
     $self->_fix_windows_nt('skip_os');
-    $self->[UA_EXTRAS] = [ @buf ];
+    $self->[UA_EXTRAS] = @buf ? [ @buf ] : undef;
     return 1;
 }
 
@@ -112,7 +110,7 @@ sub _parse_maxthon {
 
     if ( $is_30 ) {
         if ( $self->[UA_LANG] ) {
-            push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+            push @{ $self->[UA_EXTRAS] ||= [] }, $self->[UA_LANG];
             $self->[UA_LANG] = undef;
         }
     }
@@ -142,6 +140,7 @@ sub _parse_maxthon {
 sub _parse_msie {
     my($self, $moz, $thing, $extra, $name, $version) = @_;
     my $junk = shift @{ $thing }; # already used
+
     my($extras,$dotnet) = $self->_extract_dotnet( $thing, $extra );
 
     if ( @{$extras} == 2 && index( $extras->[1], 'Lunascape' ) != NO_IMATCH ) {
@@ -156,26 +155,93 @@ sub _parse_msie {
         $self->[UA_OS] = shift @{ $extras };
     }
 
+    my $real_version;
     my @buf;
     foreach my $e ( @{ $extras } ) {
-        if ( $e =~ RE_TRIDENT ) {
-            $self->[UA_TOOLKIT] = [ $1, $2 ];
+        if ( index( $e, 'Trident/' ) != NO_IMATCH ) {
+            my($tk_name, $tk_version) = split m{[/]}xms, $e, 2;
+            $self->[UA_TOOLKIT] = [ $tk_name, $tk_version ];
+            if ( $tk_name eq 'Trident' && $tk_version ) {
+                if ( $tk_version eq '7.0' && $self->[UA_VERSION_RAW] ne '11.0' ) {
+                    # more stupidity (compat mode)
+                    $self->[UA_ORIGINAL_NAME]    = 'MSIE';
+                    $self->[UA_ORIGINAL_VERSION] = 11;
+                }
+                elsif ( $tk_version eq '6.0' && $self->[UA_VERSION_RAW] ne '10.0') {
+                    # more stupidity (compat mode)
+                    $self->[UA_ORIGINAL_NAME]    = 'MSIE';
+                    $self->[UA_ORIGINAL_VERSION] = 10;
+                }
+                else {
+                    # must be the real version or some other stupidity
+                }
+            }
             next;
         }
         push @buf, $e;
     }
 
-    $self->[UA_EXTRAS] = [
-        map  { $self->trim( $_ ) }
+    my @extras =
+        map  {
+            my $thing = $self->trim( $_ );
+            lc($thing) eq 'touch'
+                ? do {
+                    $self->[UA_TOUCH]  = 1;
+                    $self->[UA_MOBILE] = 1;
+                    ();
+                  }
+                : $thing
+                ;
+        }
         grep { $_ !~ m{ \s+ compatible \z }xms }
         @buf
-    ];
+    ;
 
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
     $self->[UA_PARSER] = 'msie';
 
     return 1;
 }
 
+sub _parse_msie_11 {
+    my($self, $moz, $thing, $extra) = @_;
+
+    if ( ref $extra eq 'ARRAY' ) {
+        # remove junk
+        @{$extra} = grep { $_ ne 'like' && $_ ne 'Gecko' } @{ $extra };
+    }
+    else {
+        $extra = [];
+    }
+
+    my($version);
+    while ( my $e = shift @{ $thing } ) {
+        if (  index($e, 'rv:' ) != NO_IMATCH ) {
+            $version = (split m{rv:}xms, $e )[1] ;
+            next;
+        }
+        push @{ $extra }, $e;
+    }
+
+    $self->_parse_msie( undef, $thing, $extra, 'MSIE', $version) || return;
+
+    if ( $self->[UA_TOUCH] && $self->[UA_EXTRAS] ) {
+        # version 10+
+        my @extras = map {
+            $_ eq 'ARM'
+                ? do {
+                    $self->[UA_DEVICE] = $_;
+                    ()
+                  }
+                : $_
+        } @{ $self->[UA_EXTRAS] };
+        $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
+    }
+
+    $self->[UA_PARSER] = 'msie11';
+    return 1;
+}
+
 sub _parse_firefox {
     my($self, @args) = @_;
     $self->_parse_mozilla_family( @args );
@@ -193,6 +259,18 @@ sub _parse_firefox {
     return 1;
 }
 
+sub _parse_ff_suspect {
+    my($self, $moz, $thing, $extra, @others) = @_;
+    # fool the moz parser
+    unshift @{ $extra }, '';
+
+    $self->_parse_mozilla_family( $moz, $thing, $extra, @others );
+
+    $self->[UA_PARSER] = 'ff_suspect';
+
+    return 1;
+}
+
 sub _fix_fennec {
     my($self, $e) = @_;
     my($name, $version) = split RE_SLASH, pop @{ $e };
@@ -206,7 +284,7 @@ sub _fix_fennec {
         $self->[UA_LANG]   = undef;
     }
     elsif ( index( $self->[UA_LANG], q{ } ) != NO_IMATCH ) {
-        push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+        push @{ $self->[UA_EXTRAS] ||= [] }, $self->[UA_LANG];
         $self->[UA_LANG] = undef;
     }
     else {
@@ -260,7 +338,7 @@ sub _parse_safari {
 
         if ( $check_os && index( $check_os, 'Mac OS X' ) != NO_IMATCH ) {
             if ( $self->[UA_OS] ) {
-                push @{$self->[UA_EXTRAS]}, $self->[UA_OS];
+                push @{ $self->[UA_EXTRAS] ||= [] }, $self->[UA_OS];
             }
             $self->[UA_OS] = pop @{ $thing };
             # Another oddity: tk as "AppleWebKit/en_SG"
@@ -274,15 +352,24 @@ sub _parse_safari {
         }
     }
 
-    $self->[UA_EXTRAS] = [ @{$thing}, @others ];
+    my @extras;
+    push @extras, @{$thing}, @others;
 
     if ( $self->[UA_OS] && length($self->[UA_OS]) == 1 ) {
-        push @{$self->[UA_EXTRAS]}, $self->[UA_OS];
+        push @extras, $self->[UA_OS];
         $self->[UA_OS] = undef;
     }
 
-    push @{$self->[UA_EXTRAS]}, @junk if @junk;
-    push @{$self->[UA_EXTRAS]}, @{$extra} if $extra;
+    if ( $self->[UA_LANG] && $self->[UA_LANG] !~ m{[a-zA-Z]+}xmsg ) {
+        # some junk like "6.0" -- more stupidity
+        push @extras, $self->[UA_LANG];
+        $self->[UA_LANG] = undef;
+    }
+
+    push @extras, @junk     if @junk;
+    push @extras, @{$extra} if $extra;
+
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
 
     return 1;
 }
@@ -361,10 +448,12 @@ sub _parse_android {
         unshift @extras, join ' ', map { shift @extras } 1..3;
     }
 
+    my @extras_final = grep { $_ } @extras;
+
     $self->[UA_NAME]   = 'Android';
     $self->[UA_MOBILE] = 1;
     $self->[UA_TABLET] = $is_phone ? undef : 1;
-    $self->[UA_EXTRAS] = [ grep { $_ } @extras ];
+    $self->[UA_EXTRAS] = @extras_final ? [ @extras_final ] : undef;
 
     return 1;
 }
@@ -406,12 +495,19 @@ sub _parse_opera_pre {
     }
 
     $self->[UA_LANG] = $lang;
-    $self->[UA_OS]   = @{$thing} && $self->_is_strength( $thing->[LAST_ELEMENT] )
-                     ? shift @{$thing}
-                     : pop   @{$thing}
-                     ;
 
-    $self->[UA_EXTRAS] = [ @{ $thing }, ( $extra ? @{$extra} : () ) ];
+    if ( @{$thing} && $self->_is_strength( $thing->[LAST_ELEMENT] ) ) {
+        $self->[UA_STRENGTH] = pop   @{ $thing };
+        $self->[UA_OS]       = shift @{ $thing };
+    }
+    else {
+        $self->[UA_OS]       = pop   @{ $thing };
+    }
+
+    my @extras =  ( @{ $thing }, ( $extra ? @{$extra} : () ) );
+
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
+
     return $self->_fix_opera;
 }
 
@@ -422,11 +518,17 @@ sub _parse_opera_post {
     $self->[UA_NAME]        = shift @{$extra};
     $self->[UA_VERSION_RAW] = shift @{$extra};
    ($self->[UA_LANG]        = shift @{$extra} || q{}) =~ tr/[]//d;
-    $self->[UA_OS]          = @{$thing} && $self->_is_strength($thing->[LAST_ELEMENT])
-                            ? shift @{$thing}
-                            : pop   @{$thing}
-                            ;
-    $self->[UA_EXTRAS]      = [ @{ $thing }, ( $extra ? @{$extra} : () ) ];
+
+    if ( @{$thing} && $self->_is_strength( $thing->[LAST_ELEMENT] ) ) {
+        $self->[UA_STRENGTH] = pop   @{ $thing };
+        $self->[UA_OS]       = shift @{ $thing };
+    }
+    else {
+        $self->[UA_OS]       = pop   @{ $thing };
+    }
+
+    my @extras = ( @{ $thing }, ( $extra ? @{$extra} : () ) );
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     return $self->_fix_opera;
 }
 
@@ -443,30 +545,68 @@ sub _parse_mozilla_family {
     $self->[UA_VERSION_RAW]  = $version;
     $self->[UA_TOOLKIT]      = $extra->[0]
                              ? [ split RE_SLASH, shift @{ $extra } ]
-                             : []
+                             : undef
                              ;
 
     if ( @{$thing} && index($thing->[LAST_ELEMENT], 'rv:') != NO_IMATCH ) {
         $self->[UA_MOZILLA]  = pop @{ $thing };
-        if ( @{ $thing } <= 3 ) {
+        my $len_thing = @{ $thing };
+        if ( $len_thing == 3 ) {
             $self->[UA_OS] = shift @{ $thing };
             if ( $self->[UA_OS] && $self->[UA_OS] eq 'Macintosh' ) {
                 $self->[UA_OS] = shift @{ $thing };
             }
             $self->[UA_LANG] = pop @{ $thing } if @{ $thing };
         }
+        elsif ( $len_thing <= 2 ) {
+            if (   $thing->[0] eq 'X11'
+                || index( $thing->[-1], 'Intel' ) != NO_IMATCH
+            ) {
+                if ( index( lc $thing->[-1], 'linux arm') != NO_IMATCH ) {
+                    $self->[UA_DEVICE] = pop @{ $thing };
+                    $self->[UA_OS]     = 'Linux'; # Android? huh?
+                }
+                else {
+                    $self->[UA_OS]   = pop @{ $thing };
+                }
+            }
+            elsif (
+                   index( lc $thing->[0], 'android' ) != NO_IMATCH
+                || index( lc $thing->[0], 'maemo'   ) != NO_IMATCH
+            ) {
+                # mobile? tablet?
+                $self->[UA_OS]     = shift @{ $thing };
+                $self->[UA_DEVICE] = shift @{ $thing };
+                if ( lc $self->[UA_DEVICE] eq 'tablet' ) {
+                    $self->[UA_TABLET] = 1;
+                }
+            }
+            else {
+                if ( $len_thing > 1 ) {
+                    if ( $thing->[-1] ne 'WOW64' ) {
+                        $self->[UA_LANG] = pop @{ $thing };
+                    }
+                }
+                else {
+                    $self->[UA_OS]   = pop @{ $thing };
+                }
+            }
+        }
         else {
+
             $self->[UA_LANG]     = pop @{ $thing };
             $self->[UA_OS]       = pop @{ $thing };
         }
     }
 
-    $self->[UA_EXTRAS] = [
-        grep { $_ }
+    my @extras = grep { $_ }
         @{ $thing },
         @others,
         $extra ? @{ $extra } : (),
-    ];
+    ;
+
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
+
     return 1;
 }
 
@@ -510,13 +650,13 @@ sub _parse_gecko {
             push @buf, $e;
         }
 
-        $self->[UA_EXTRAS]        = [ @buf ];
+        $self->[UA_EXTRAS]        = @buf ? [ @buf ] : undef;
         $self->[UA_ORIGINAL_NAME] = $before if $before ne $self->[UA_NAME];
         $self->_fix_windows_nt;
         return 1 ;
     }
 
-    if ( $self->[UA_TOOLKIT] && $self->[UA_TOOLKIT][TK_NAME] eq 'Gecko' ) {
+    if ( ref $self->[UA_TOOLKIT] eq 'ARRAY' && $self->[UA_TOOLKIT][TK_NAME] eq 'Gecko' ) {
         ($self->[UA_NAME], $self->[UA_VERSION_RAW]) = split RE_SLASH, $moz;
         if ( $self->[UA_NAME] && $self->[UA_VERSION_RAW] ) {
             $self->[UA_PARSER] = 'mozilla_family:gecko';
@@ -542,10 +682,12 @@ sub _fix_windows_nt {
     my $os      = $self->[UA_OS] || q{};
     return if ( ! $os              && ! $skip_os )
         ||    (   $os ne 'windows' && ! $skip_os )
+        ||    ref $self->[UA_EXTRAS] ne 'ARRAY'
         ||      ! $self->[UA_EXTRAS][0]
-        ||        $self->[UA_EXTRAS][0] !~ m{ NT\s?(\d.*?) \z }xmsi;
+        ||        $self->[UA_EXTRAS][0] !~ m{ NT\s?(\d.*?) \z }xmsi
+    ;
     $self->[UA_EXTRAS][0] = $self->[UA_OS]; # restore
-    $self->[UA_OS] = "Windows NT $1"; # fix
+    $self->[UA_OS]        = "Windows NT $1"; # fix
     return;
 }
 
@@ -564,7 +706,7 @@ sub _parse_netscape {
     $self->[UA_VERSION_RAW] = $version;
     $self->[UA_OS]          = $buf[0] eq 'X11' ? pop @buf : shift @buf;
     $self->[UA_NAME]        = 'Netscape';
-    $self->[UA_EXTRAS]      = [ @buf ];
+    $self->[UA_EXTRAS]      = @buf ? [ @buf ] : undef;
     if ( $junk ) {
         $junk =~ s{ \[ (.+?) \] .* \z}{$1}xms;
         $self->[UA_LANG] = $junk if $junk;
@@ -579,6 +721,21 @@ sub _generic_moz_thing {
     my($mname, $mversion, @rest) = split RE_CHAR_SLASH_WS, $moz;
     return if $mname eq 'Mozilla' || $mname eq 'Emacs-W3';
 
+    if ( index( $mname, 'Nokia' ) != NO_IMATCH ) {
+        my($device, $num, $os, $series, @junk) = split m{[\s]+}xms,
+                                                    $self->[UA_STRING_ORIGINAL];
+        if (   $device
+            && $num
+            && $os
+            && $series
+            && index( $os, 'SymbianOS' ) != NO_IMATCH
+        ) {
+            return $self->_parse_symbian(
+                        join ';', $os, "$series $device", join(q{ }, @junk, $num)
+                    );
+        }
+    }
+
     $self->[UA_NAME]        = $mname;
     $self->[UA_VERSION_RAW] = $mversion || ( $mname eq 'Links' ? shift @{$t} : 0 );
     $self->[UA_OS] = @rest                                     ? join(q{ }, @rest)
@@ -590,7 +747,7 @@ sub _generic_moz_thing {
         \$self->[UA_OS], \$self->[UA_NAME], \$self->[UA_VERSION_RAW], \@extras
     );
 
-    $self->[UA_EXTRAS]      = [ @extras ] if @extras;
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     $self->[UA_GENERIC]     = 1;
     $self->[UA_PARSER]      = 'generic_moz_thing';
 
@@ -637,8 +794,8 @@ sub _generic_compatible {
         if ( $self->_is_generic_bogus_ie( $extra ) ) {
             # edge case
             my($n, $v) = split RE_WHITESPACE, shift @orig_thing;
-            my $e = [ split RE_SC_WS, join q{ }, @{ $extra } ];
-            my $t = \@orig_thing;
+            my $e      = [ split RE_SC_WS, join q{ }, @{ $extra } ];
+            my $t      = \@orig_thing;
             push @{ $e }, grep { $_ } map { split RE_SC_WS, $_ } @others;
             $self->_parse_msie( $moz, $thing, $e, $n, $v );
             return 1;
@@ -656,13 +813,22 @@ sub _generic_compatible {
 
     @extras = (@{$thing}, $extra ? @{$extra} : (), @others ) if ! @extras;
 
+    if ( $lang && index( $lang, 'MSIE ') != NO_IMATCH ) {
+        return $self->_parse_msie(
+                    $moz,
+                    [],
+                    [$os, "$name/$version", @extras], # junk
+                    split( m{[\s]+}xms, $lang, 2 ),   # name, version
+                );
+    }
+
     $self->_fix_generic( \$os, \$name, \$version, \@extras );
 
     $self->[UA_NAME]        = $name;
     $self->[UA_VERSION_RAW] = $version || 0;
     $self->[UA_OS]          = $os;
     $self->[UA_LANG]        = $lang;
-    $self->[UA_EXTRAS]      = [ @extras ] if @extras;
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     $self->[UA_GENERIC]     = 1;
     $self->[UA_PARSER]      = 'generic_compatible';
 
@@ -681,24 +847,56 @@ sub _parse_emacs {
     my @rest = (  @{ $thing }, @moz );
     push @rest, @{ $extra } if $extra && ref $extra eq 'ARRAY';
     push @rest, ( map { split RE_SC_WS, $_ } @others ) if @others;
-    $self->[UA_EXTRAS]      = [ grep { $_ } map { $self->trim( $_ ) } @rest ];
+    my @extras = grep { $_ } map { $self->trim( $_ ) } @rest;
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     $self->[UA_PARSER]      = 'emacs';
     return 1;
 }
 
 sub _parse_moz_only {
-    my($self, $moz) = @_;
+    my $self  = shift;
+    my($moz)  = @_;
     my @parts = split RE_WHITESPACE, $moz;
     my $id = shift @parts;
     my($name, $version) = split RE_SLASH, $id;
+
+    if ( index( $name, 'Symbian' ) != NO_IMATCH ) {
+        return $self->_parse_symbian( $moz );
+    }
+
     if ( $name eq 'Mozilla' && @parts ) {
         ($name, $version) = split RE_SLASH, shift @parts;
         return if ! $name || ! $version;
     }
+
     $self->[UA_NAME]        = $name;
     $self->[UA_VERSION_RAW] = $version || 0;
-    $self->[UA_EXTRAS]      = [ @parts ];
+    $self->[UA_EXTRAS]      = @parts ? [ @parts ] : undef;
     $self->[UA_PARSER]      = 'moz_only';
+    $self->[UA_ROBOT]       = 1 if ! $self->[UA_VERSION_RAW];
+
+    return 1;
+}
+
+sub _parse_symbian {
+    my($self, $raw) = @_;
+    my($os, $series_device, @rest) = split m{[;]\s?}xms, $raw;
+
+    return if ! $os || ! $series_device;
+
+    my($series, $device) = split m{[\s]+}xms, $series_device;
+
+    return if ! $device;
+
+    my @extras = map { split m{[\s]+}xms, $_ } @rest;
+
+    @{ $self }[ UA_NAME, UA_VERSION_RAW ] = split RE_SLASH, $series, 2;
+    $self->[UA_OS]     = $os;
+    $self->[UA_DEVICE] = $device;
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
+    $self->[UA_MOBILE] = 1;
+    $self->[UA_PARSER] = 'symbian';
+
     return 1;
 }
 
@@ -714,7 +912,7 @@ sub _parse_hotjava {
             @parts = map { $self->trim( $_ ) } @parts;
             $self->[UA_OS]     = pop @parts;
             $self->[UA_LANG]   = pop @parts;
-            $self->[UA_EXTRAS] = [ @parts ];
+            $self->[UA_EXTRAS] = @parts ? [ @parts ] : undef;
         }
     }
     return 1;
@@ -726,7 +924,7 @@ sub _parse_docomo {
         my($name, $version)     = split RE_SLASH, shift @{ $thing };
         $self->[UA_NAME]        = $name;
         $self->[UA_VERSION_RAW] = $version;
-        $self->[UA_EXTRAS]      = [ @{ $thing } ];
+        $self->[UA_EXTRAS]      = @{ $thing } > 0 ? [ @{ $thing } ] : undef;
         $self->[UA_MOBILE]      = 1;
         $self->[UA_ROBOT]       = 1;
         $self->[UA_PARSER]      = 'docomo';
@@ -743,32 +941,37 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Parse::HTTP::UserAgent::Base::Parsers - Base class
+Parse::HTTP::UserAgent::Base::Parsers
+
+=head1 VERSION
+
+version 0.41
 
 =head1 DESCRIPTION
 
-This document describes version C<0.39> of C<Parse::HTTP::UserAgent::Base::Parsers>
-released on C<2 December 2013>.
-
 Internal module.
 
+=head1 NAME
+
+Parse::HTTP::UserAgent::Base::Parsers - Base class
+
 =head1 SEE ALSO
 
 L<Parse::HTTP::UserAgent>.
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak Gursoy <burak@cpan.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 - 2013 Burak Gursoy. All rights reserved.
+This software is copyright (c) 2009 by Burak Gursoy.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.16.2 or,
-at your option, any later version of Perl 5 you may have available.
 =cut
